package cn.com.qmth.print.manage.controller;

import cn.com.qmth.print.manage.entity.UserEntity;
import cn.com.qmth.print.manage.service.PmSession;
import cn.com.qmth.print.manage.service.UserService;
import cn.com.qmth.print.manage.service.query.UserQuery;

import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonNullFormatVisitor;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.StatusException;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

/**
 * @Date: 2021/11/16.
 */
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/user")
@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
public class UserController extends BaseController {

    @Autowired
    private UserService userService;

    /**
     * 用户分页查询
     *
     * @param query
     * @return
     */
    @RequestMapping(value = "/page", method = RequestMethod.POST)
    public Object page(@Validated UserQuery query) {
        return userService.pageQuery(query);
    }

    /**
     * 用户查询
     *
     * @return
     */
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Object list() {
        return userService.listByEnable();
    }

    /**
     * 新增/修改用户
     *
     * @param user
     * @return
     */
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Object save(@RequestAttribute PmSession accessEntity, UserEntity user) {
        Long userId = getAccessUserId(accessEntity);
        if (user.getId() != null) {
            UserEntity userEntity = userService.getById(user.getId());
            if (userEntity == null) {
                throw new StatusException("用户不存在");
            }
            if (!user.getLoginName().equals(userEntity.getLoginName())) {
                throw new StatusException("用户名不能修改");
            }
        }
        if(StringUtils.isBlank(user.getName())){
            user.setName(user.getLoginName());
        }
        user.setCreatorId(userId);
        user.setCreateTime(new Date());
        user.setUpdaterId(userId);
        user.setUpdateTime(new Date());
        return userService.saveOrUpdate(user);
    }
}
