package cn.com.qmth.print.manage.service.impl;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.print.manage.dao.BreakRecordDao;
import cn.com.qmth.print.manage.entity.BreakRecordEntity;
import cn.com.qmth.print.manage.entity.ExamEntity;
import cn.com.qmth.print.manage.entity.ExamStudentEntity;
import cn.com.qmth.print.manage.entity.UserEntity;
import cn.com.qmth.print.manage.enums.GroupType;
import cn.com.qmth.print.manage.enums.RecordStatus;
import cn.com.qmth.print.manage.service.BreakRecordService;
import cn.com.qmth.print.manage.service.ExamService;
import cn.com.qmth.print.manage.service.ExamStudentService;
import cn.com.qmth.print.manage.service.UserService;
import cn.com.qmth.print.manage.service.query.RecordQuery;
import cn.com.qmth.print.manage.vo.BreakRecordVo;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.StatusException;

@Service
public class BreakRecordServiceImpl extends ServiceImpl<BreakRecordDao, BreakRecordEntity> implements
        BreakRecordService {

    @Autowired
    private BreakRecordDao breakRecordDao;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private ExamService examService;

    @Autowired
    private UserService userService;

    @Override
    public BreakRecordVo start(Long userId, Long examId, String startNumber) {
        ExamEntity exam = examService.getById(examId);
        if (exam == null) {
            throw new StatusException("找不到考试");
        }
        ExamStudentEntity student = studentService.findByExamIdAndExamNumber(examId, startNumber);
        if (student == null) {
            throw new StatusException("找不到考生");
        }
        String groupName = null;
        if (GroupType.COURSE.equals(exam.getGroupType())) {
            groupName = student.getCourseCode();
        } else if (GroupType.EXAM_SITE.equals(exam.getGroupType())) {
            groupName = student.getExamSite();
        } else if (GroupType.EXAM_ROOM.equals(exam.getGroupType())) {
            groupName = student.getExamRoom();
        }
        BreakRecordEntity recordEntity = findByExamIdAndStartNumberAndStatus(examId, startNumber, RecordStatus.ERROR);
        if (recordEntity == null) {
            recordEntity = new BreakRecordEntity();
            recordEntity.setOrgId(exam.getOrgId());
            recordEntity.setExamId(examId);
            recordEntity.setCreatorId(userId);
            recordEntity.setCreateTime(new Date());
            recordEntity.setStartNumber(startNumber);
            recordEntity.setGroupType(exam.getGroupType());
            recordEntity.setGroupName(groupName);
            recordEntity.setStatus(RecordStatus.ERROR);
            saveOrUpdate(recordEntity);
        }
        return new BreakRecordVo(recordEntity, student);
    }

    @Override
    public BreakRecordEntity findByExamIdAndStartNumberAndStatus(Long examId, String startNumber, RecordStatus status) {
        QueryWrapper<BreakRecordEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BreakRecordEntity::getExamId, examId);
        queryWrapper.lambda().eq(BreakRecordEntity::getStartNumber, startNumber);
        queryWrapper.lambda().eq(BreakRecordEntity::getStatus, status);
        return this.getOne(queryWrapper);
    }

    @Override
    public BreakRecordVo end(Long id, Long userId, Long examId, String endNumber) {
        ExamEntity exam = examService.getById(examId);
        if (exam == null) {
            throw new StatusException("找不到考试");
        }
        ExamStudentEntity student = studentService.findByExamIdAndExamNumber(examId, endNumber);
        if (student == null) {
            throw new StatusException("找不到考生");
        }

        BreakRecordEntity recordEntity = this.getById(id);
        if (recordEntity == null) {
            throw new StatusException("找不到中断记录");
        }
        ExamStudentEntity start = studentService.findByExamIdAndExamNumber(examId, recordEntity.getStartNumber());
        if (student.getSortNo() != start.getSortNo() + 1) {
            throw new StatusException("中断结束考生不正确");
        }
        recordEntity.setEndNumber(endNumber);
        recordEntity.setStatus(RecordStatus.NORMAL);
        recordEntity.setUpdaterId(userId);
        recordEntity.setUpdateTime(new Date());
        saveOrUpdate(recordEntity);
        return new BreakRecordVo(recordEntity, student);
    }

    @Override
    public IPage<BreakRecordEntity> pageQuery(RecordQuery query) {
        QueryWrapper<BreakRecordEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BreakRecordEntity> lw = wrapper.lambda();
        if (query.getStatus() != null) {

            lw.eq(BreakRecordEntity::getStatus, query.getStatus());
        }
        if (query.getGroupType() != null) {
            lw.eq(BreakRecordEntity::getGroupType, query.getGroupType());
        }
        lw.eq(BreakRecordEntity::getExamId, query.getExamId());
        lw.orderByAsc(BreakRecordEntity::getId);
        Page<BreakRecordEntity> page = new Page<>(query.getPageNumber(), query.getPageSize());
        Page<BreakRecordEntity> entityPage = breakRecordDao.selectPage(page, wrapper);
        for (BreakRecordEntity record : entityPage.getRecords()) {
            if (record.getUpdaterId() != null) {
                UserEntity userEntity = userService.getById(record.getUpdaterId());
                record.setUserName(userEntity.getName());
            }
        }
        return entityPage;
    }

    @Override
    public List<BreakRecordEntity> listByExamId(Long examId) {
        QueryWrapper<BreakRecordEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BreakRecordEntity::getExamId, examId);
        return this.list(queryWrapper);
    }

    @Override
    public void deleteByExamId(Long examId) {
        UpdateWrapper<BreakRecordEntity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(BreakRecordEntity::getExamId, examId);
        this.remove(updateWrapper);
    }

    @Override
    public BreakRecordVo findOne(Long id, String examNumber) {
        BreakRecordEntity entity = this.getById(id);
        if (entity == null) {
            throw new StatusException("找不到中断记录");
        }
        ExamStudentEntity student = studentService.findByExamIdAndExamNumber(entity.getExamId(), examNumber);
        if (student == null) {
            throw new StatusException("找不到考生");
        }
        return new BreakRecordVo(entity, student);
    }
}
